==> thirdparty-service-biz-1.0.33.p2-20211015.034711-1-sources.jar: cn/com/duiba/service/HttpAsyncClientPool.java <==
package cn.com.duiba.service;

import cn.com.duiba.biz.credits.CgbBankApi;
import cn.com.duiba.biz.credits.CiticBankApi;
import cn.com.duiba.biz.credits.GangZhongLvApi;
import cn.com.duiba.biz.credits.SnsApi;
import cn.com.duiba.biz.credits.WjrcbApi;
import cn.com.duiba.domain.RequestTypeEnum;
import cn.com.duiba.tool.UrlUtils;
import cn.com.duiba.wolf.utils.NumberUtils;
import cn.com.duibaboot.ext.autoconfigure.core.utils.CatUtils;
import cn.com.duibaboot.ext.autoconfigure.httpclient.ssre.CanAccessInsideNetwork;
import com.alibaba.ttl.threadpool.TtlExecutors;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.concurrent.FutureCallback;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * HTTP异步请求池
 */
@Service
public class HttpAsyncClientPool implements InitializingBean, DisposableBean {
	private static final Logger log = LoggerFactory.getLogger(HttpAsyncClientPool.class);

	/**
	 * APP最大处理队列
	 */
	public static final int MAX_APP_QUEUE = 1000;

	@Resource(name="slowHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient slowHttpAsyncClient;

	@Resource(name="httpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient httpAsyncClient;

	@Resource(name="virtualHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient virtualHttpAsyncClient;

	@Resource(name="addHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient addHttpAsyncClient;

    @Resource(name="citicBankHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient citicBankHttpAsyncClient;

	@Resource(name="wjrcbHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient wjrcbHttpAsyncClient;

	@Resource(name="citicBankXiAnHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient citicBankXianHttpAsyncClient;

	private Map<String, AtomicInteger> runningStat = new ConcurrentHashMap<>();

	@Autowired
	private ThreadPoolService threadPoolService;

	//港中旅单独定制
	@Resource(name = "ctsExecutorService")
	private ExecutorService ctsExecutorService;

    @Resource(name = "ctsHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient ctsHttpAsyncClient;

    @Resource(name="cgbHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient cgbHttpAsyncClient;

	@Autowired
	private SlowRequestHandler slowRequestHandler;

	@Autowired
    private CiticBankApi citicBankApi;
	@Autowired
	private WjrcbApi wjrcbApi;

	@Autowired
	private SnsApi snsApi;

	@Autowired
    private GangZhongLvApi gangZhongLvApi;

	@Autowired
    private CgbBankApi cgbBankApi;

	private static final int MAX_QUEUE_SIZE = 50000;
	private final BlockingQueue<CallbackProcesser> latterQueue = new LinkedBlockingQueue<>(MAX_QUEUE_SIZE);
	private final ExecutorService executorService = TtlExecutors.getTtlExecutorService(Executors.newSingleThreadExecutor(new ThreadFactory() {
		@Override
		public Thread newThread(Runnable r) {
			return new Thread(r, "LatterQueueProcess");
		}
	}));
	private volatile boolean status = true;

	/**
	 * 执行HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	private void execute(String queueKey, HttpUriRequest request, FutureCallback<HttpResponse> callback, CloseableHttpAsyncClient httpClient) {
		CallbackProcesser process = null;
		try {
			long s = System.currentTimeMillis();
			AtomicInteger running = runningStat.get(queueKey);
			if (running == null) {
				runningStat.put(queueKey, new AtomicInteger(1));
			} else {
				running.incrementAndGet();
			}
            process = new CallbackProcesser(queueKey, request, callback);
			request.setHeader("User-Agent","Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1");

			String reqUrl = UrlUtils.buildUrl(request.getURI());
			if(slowRequestHandler.isBlackHost(reqUrl)){
				log.info("{} is black host", reqUrl);
				return;
			}
            if(executeCustom(queueKey, request, httpClient, s, process)) { // 定制执行
                return;
            }
			if (slowRequestHandler.isSlow(reqUrl)) {
				log.info("url [{}], add to queue, appId={}", reqUrl, queueKey);
				slowRequestHandler.addQueue(process);
				return;
			}
			httpClient.execute(request, process);
			long e = System.currentTimeMillis();
			long t = e - s;
			if (t > 1000) {
				log.warn("http get pool time > {} ms for {}, appId={}", t, request.getURI(), queueKey);
				if (t>2000) {
					slowRequestHandler.addSlow(reqUrl, (int)(t/1000L));
				}
			}
		} catch (Exception e) {
			log.error("execute:", e);
			if(e.getCause() != null){
                log.error("cause:", e.getCause());
            }
			if(process != null){
				process.failed(e);
			}
		}
	}

    /**
     * 定制执行
     * @param queueKey
     * @param request
     * @param httpClient
     * @param s
     * @param process
     * @return
     */
	private boolean executeCustom(String queueKey, HttpUriRequest request, CloseableHttpAsyncClient httpClient, long s, CallbackProcesser process) {
        if(citicBankApi.isCiticBank(NumberUtils.parseLong(queueKey, 0L)) || citicBankApi.isXiAnApp(NumberUtils.parseLong(queueKey, 0L))) {
            httpClient.execute(request, process);
            long e = System.currentTimeMillis();
            long t = e - s;
            if(t > 1000) {
                log.warn("http get pool time > {} ms for {}, appId={}", t, request.getURI(), queueKey);
            }
            return true;
        }
        return false;
    }

	/**
	 * 提交HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	public void submit(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
		submit(queueKey, request, callback, RequestTypeEnum.SUB_CREDITS);
	}

	/**
	 * 提交HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	public void submit(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback, RequestTypeEnum requestTypeEnum) {
        if(submitCustom(queueKey, request, callback)) { // 定制提交
            return;
        }
	    CloseableHttpAsyncClient httpClient = getCloseableHttpAsyncClient(requestTypeEnum);
		if (threadPoolService.canSubmit()) {
			threadPoolService.submit(new Runnable() {
				@Override
				public void run() {
					execute(queueKey, request, callback, httpClient);
				}
			});
		} else {
			try {
				latterQueue.add(new CallbackProcesser(queueKey, request, callback));
			} catch (IllegalStateException e) {
				// 队列满了,任务取消
				log.warn("latterQueue is temporarily full",e);
				callback.cancelled();
			}
		}
	}

    /**
     * 定制提交
     * @param queueKey
     * @param request
     * @param callback
     * @return
     */
	private boolean submitCustom(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
        Long appId = NumberUtils.parseLong(queueKey, 0L);
        CloseableHttpAsyncClient customHttpClient = getCustomHttpClient(appId);
        if(customHttpClient != null) {
            //港中旅定制线程池
            if (gangZhongLvApi.isGangZhongLv(appId)) {
                ctsExecutorService.submit(() -> execute(queueKey, request, callback, customHttpClient));
            } else if (threadPoolService.canSubmit()) {
                threadPoolService.submit(() -> execute(queueKey, request, callback, customHttpClient));
            } else {
                execute(queueKey, request, callback, customHttpClient);
            }
            return true;
        }
        return false;
    }

    /**
     * 获取定制的httpClient
     * @param appId
     * @return
     */
    private CloseableHttpAsyncClient getCustomHttpClient(Long appId) {
        if(citicBankApi.isCiticBank(appId)) {
            return citicBankHttpAsyncClient;
        }
        if (citicBankApi.isXiAnApp(appId)) {
        	return citicBankXianHttpAsyncClient;
		}
        if (wjrcbApi.isWjrcb(appId)) {
        	return wjrcbHttpAsyncClient;
		}
        if(gangZhongLvApi.isGangZhongLv(appId)){
            return ctsHttpAsyncClient;
        }
        if (cgbBankApi.isCgbBank(appId)) {
            return cgbHttpAsyncClient;
        }
        return null;
    }

	@NotNull
	private CloseableHttpAsyncClient getCloseableHttpAsyncClient(RequestTypeEnum requestTypeEnum) {
		CloseableHttpAsyncClient httpClient = httpAsyncClient;
		if(requestTypeEnum != null && requestTypeEnum.getCode() == RequestTypeEnum.VIRTUAL.getCode()){
			httpClient = virtualHttpAsyncClient;
		}
		if(requestTypeEnum != null && requestTypeEnum.getCode() == RequestTypeEnum.ADD_CREDITS.getCode()){
			httpClient = addHttpAsyncClient;
		}
		return httpClient;
	}

	/**
	 * 判断APP能否提交任务到执行队列中
	 *
	 * @param queueKey
	 * @return
	 */
	public boolean canSubmitToAppPool(String queueKey) {
		int queueCount = 0;
		AtomicInteger running = runningStat.get(queueKey);
		if (running != null) {
			queueCount = running.intValue();
		}
		if (queueCount < MAX_APP_QUEUE) {
			return true;
		}
		return false;
	}

	/**
	 * 队列详细信息
	 *
	 * @return
	 */
	public Map<String, Object> dumpDetail() {
		Map<String, Object> map = new HashMap<>();
		for (Map.Entry<String, AtomicInteger> entry : runningStat.entrySet()) {
			if (entry.getValue().intValue() > 0) {
				map.put(entry.getKey(), entry.getValue());
			}
		}
		Map<String, Object> ret = new HashMap<>();
		ret.put("runningStat", map);
		return ret;
	}

	/**
	 * HTTP处理中的个数
	 *
	 * @return
	 */
	public int dumpSize() {
		int running = 0;
		for (AtomicInteger a : runningStat.values()) {
			running += a.get();
		}
		return running;
	}

	/**
	 * 自定义回调
	 */
	public class CallbackProcesser implements FutureCallback<HttpResponse> {

		private FutureCallback<HttpResponse> callback;
		private String queueKey;
		private HttpUriRequest request;

		public CallbackProcesser(String queueKey, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
			this.callback = callback;
			this.queueKey = queueKey;
			this.request = request;
		}

		@Override
		public void completed(HttpResponse result) {
			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.completed(result);
					return null;
				}, "AsyncHttp", "completed");
			}catch (Throwable te ){
				log.error("completed;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//
//			try {
//				callback.completed(result);
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		@Override
		public void failed(Exception ex) {

			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.failed(ex);
					return null;
				}, "AsyncHttp", "failed");
			} catch (Throwable te) {
				log.error("failed;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//			try {
//				callback.failed(ex);
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		@Override
		public void cancelled() {

			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.cancelled();
					return null;
				}, "AsyncHttp", "cancelled");
			} catch (Throwable te) {
				log.error("cancelled;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//			try {
//				callback.cancelled();
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		public HttpUriRequest getRequest() {
			return request;
		}

		public String getQueueKey() {
			return queueKey;
		}
	}

	/**
	 * 销毁关闭
	 */
	@Override
	public void destroy() throws Exception {
		status = false;
		executorService.shutdown();
	}

	/**
	 * 初始化启动
	 */
	@Override
	public void afterPropertiesSet() throws Exception {
		executorService.execute(new Runnable() {
			@Override
			public void run() {
				while (status) {
					try{
						if (!threadPoolService.canSubmit()) {
							TimeUnit.MILLISECONDS.sleep(200);
							continue;
						}
						CallbackProcesser callbackProcesser =latterQueue.take();
						if (callbackProcesser == null) {
							continue;
						}
						final String queueKey = callbackProcesser.getQueueKey();
						final HttpUriRequest request = callbackProcesser.getRequest();
						final FutureCallback<HttpResponse> callback = callbackProcesser.callback;
						threadPoolService.submit(new Runnable() {
							@Override
							public void run() {
								execute(queueKey, request, callback, slowHttpAsyncClient);
							}
						});
					}catch (InterruptedException e) {
						log.warn("Interrupted",e);
					}catch (Exception e) {
						log.warn("",e);
					}
				}
			}
		});
	}
}

==> thirdparty-service-biz-1.0.45-20220111.060650-1-sources.jar: cn/com/duiba/service/HttpAsyncClientPool.java <==
package cn.com.duiba.service;

import cn.com.duiba.biz.credits.CgbBankApi;
import cn.com.duiba.biz.credits.CiticBankApi;
import cn.com.duiba.biz.credits.GangZhongLvApi;
import cn.com.duiba.biz.credits.WjrcbApi;
import cn.com.duiba.constant.hsbc.HsbcConfig;
import cn.com.duiba.domain.RequestTypeEnum;
import cn.com.duiba.tool.UrlUtils;
import cn.com.duiba.wolf.utils.NumberUtils;
import cn.com.duibaboot.ext.autoconfigure.core.utils.CatUtils;
import cn.com.duibaboot.ext.autoconfigure.httpclient.ssre.CanAccessInsideNetwork;
import com.alibaba.ttl.threadpool.TtlExecutors;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.concurrent.FutureCallback;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * HTTP异步请求池
 */
@Service
public class HttpAsyncClientPool implements InitializingBean, DisposableBean {
	private static final Logger log = LoggerFactory.getLogger(HttpAsyncClientPool.class);

	/**
	 * APP最大处理队列
	 */
	public static final int MAX_APP_QUEUE = 1000;

	@Resource(name="slowHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient slowHttpAsyncClient;

	@Resource(name="httpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient httpAsyncClient;

	@Resource(name="virtualHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient virtualHttpAsyncClient;

	@Resource(name="addHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient addHttpAsyncClient;

    @Resource(name="citicBankHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient citicBankHttpAsyncClient;

	@Resource(name="wjrcbHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient wjrcbHttpAsyncClient;

	@Resource(name="citicBankXiAnHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient citicBankXianHttpAsyncClient;

	private Map<String, AtomicInteger> runningStat = new ConcurrentHashMap<>();

	@Autowired
	private ThreadPoolService threadPoolService;

	//港中旅单独定制
	@Resource(name = "ctsExecutorService")
	private ExecutorService ctsExecutorService;

    @Resource(name = "ctsHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient ctsHttpAsyncClient;

    @Resource(name="cgbHttpAsyncClient")
    @CanAccessInsideNetwork
    private CloseableHttpAsyncClient cgbHttpAsyncClient;

	@Resource(name="xiangShangHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient xiangShangHttpAsyncClient;


	@Resource(name = "hsbcHttpAsyncClient")
	@CanAccessInsideNetwork
	private CloseableHttpAsyncClient hsbcHttpAsyncClient;


	@Resource
	private HsbcConfig hsbcConfig;

	@Autowired
	private SlowRequestHandler slowRequestHandler;

	@Autowired
    private CiticBankApi citicBankApi;
	@Autowired
	private WjrcbApi wjrcbApi;

	@Autowired
    private GangZhongLvApi gangZhongLvApi;

	@Autowired
    private CgbBankApi cgbBankApi;


	private static final int MAX_QUEUE_SIZE = 50000;
	private final BlockingQueue<CallbackProcesser> latterQueue = new LinkedBlockingQueue<>(MAX_QUEUE_SIZE);
	private final ExecutorService executorService = TtlExecutors.getTtlExecutorService(Executors.newSingleThreadExecutor(new ThreadFactory() {
		@Override
		public Thread newThread(Runnable r) {
			return new Thread(r, "LatterQueueProcess");
		}
	}));
	private volatile boolean status = true;

	/**
	 * 执行HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	private void execute(String queueKey, HttpUriRequest request, FutureCallback<HttpResponse> callback, CloseableHttpAsyncClient httpClient) {
		CallbackProcesser process = null;
		try {
			request.setHeader("User-Agent","Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1");
			String reqUrl = UrlUtils.buildUrl(request.getURI());
			if(StringUtils.isBlank(reqUrl) || slowRequestHandler.isBlackHost(reqUrl)){
				log.info("{} is black host", reqUrl);
				return;
			}

			long s = System.currentTimeMillis();
			AtomicInteger running = runningStat.get(queueKey);
			if (running == null) {
				runningStat.put(queueKey, new AtomicInteger(1));
			} else {
				running.incrementAndGet();
			}

            process = new CallbackProcesser(queueKey, request, callback);
            if(executeCustom(queueKey, request, httpClient, s, process)) { // 定制执行
                return;
            }
			if (slowRequestHandler.isSlow(reqUrl)) {
				log.info("url [{}], add to queue, appId={}", reqUrl, queueKey);
				slowRequestHandler.addQueue(process);
				return;
			}
			httpClient.execute(request, process);
			long e = System.currentTimeMillis();
			long t = e - s;
			if (t > 1000) {
				log.warn("http get pool time > {} ms for {}, appId={}", t, request.getURI(), queueKey);
				if (t>2000) {
					slowRequestHandler.addSlow(reqUrl, (int)(t/1000L));
				}
			}
		} catch (Exception e) {
			log.error("execute:", e);
			if(e.getCause() != null){
                log.error("cause:", e.getCause());
            }
			if(process != null){
				process.failed(e);
			}
		}
	}

    /**
     * 定制执行
     * @param queueKey
     * @param request
     * @param httpClient
     * @param s
     * @param process
     * @return
     */
	private boolean executeCustom(String queueKey, HttpUriRequest request, CloseableHttpAsyncClient httpClient, long s, CallbackProcesser process) {
        if(citicBankApi.isCiticBank(NumberUtils.parseLong(queueKey, 0L)) || citicBankApi.isXiAnApp(NumberUtils.parseLong(queueKey, 0L))) {
            httpClient.execute(request, process);
            long e = System.currentTimeMillis();
            long t = e - s;
            if(t > 1000) {
                log.warn("http get pool time > {} ms for {}, appId={}", t, request.getURI(), queueKey);
            }
            return true;
        }
        return false;
    }

	/**
	 * 提交HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	public void submit(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
		submit(queueKey, request, callback, RequestTypeEnum.SUB_CREDITS);
	}

	/**
	 * 提交HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	public void submitForXiangShang(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
		if (threadPoolService.canSubmit()) {
			threadPoolService.submit(() -> execute(queueKey, request, callback, xiangShangHttpAsyncClient));
		} else {
			execute(queueKey, request, callback, xiangShangHttpAsyncClient);
		}
	}

	/**
	 * 提交HTTP异步请求
	 *
	 * @param queueKey
	 * @param request
	 * @param callback
	 */
	public void submit(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback, RequestTypeEnum requestTypeEnum) {
        if(submitCustom(queueKey, request, callback)) { // 定制提交
            return;
        }
	    CloseableHttpAsyncClient httpClient = getCloseableHttpAsyncClient(requestTypeEnum);
		if (threadPoolService.canSubmit()) {
			threadPoolService.submit(new Runnable() {
				@Override
				public void run() {
					execute(queueKey, request, callback, httpClient);
				}
			});
		} else {
			try {
				latterQueue.add(new CallbackProcesser(queueKey, request, callback));
			} catch (Exception e) {
				// 队列满了,任务取消
				log.warn("latterQueue is temporarily full",e);
				callback.cancelled();
			}
		}
	}

    /**
     * 定制提交
     * @param queueKey
     * @param request
     * @param callback
     * @return
     */
	private boolean submitCustom(final String queueKey, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
        Long appId = NumberUtils.parseLong(queueKey, 0L);
        CloseableHttpAsyncClient customHttpClient = getCustomHttpClient(appId);
        if(customHttpClient != null) {
            //港中旅定制线程池
            if (gangZhongLvApi.isGangZhongLv(appId)) {
                ctsExecutorService.submit(() -> execute(queueKey, request, callback, customHttpClient));
            } else if (threadPoolService.canSubmit()) {
                threadPoolService.submit(() -> execute(queueKey, request, callback, customHttpClient));
            } else {
                execute(queueKey, request, callback, customHttpClient);
            }
            return true;
        }
        return false;
    }

    /**
     * 获取定制的httpClient
     * @param appId
     * @return
     */
    private CloseableHttpAsyncClient getCustomHttpClient(Long appId) {
        if(citicBankApi.isCiticBank(appId)) {
            return citicBankHttpAsyncClient;
        }
        if (citicBankApi.isXiAnApp(appId)) {
        	return citicBankXianHttpAsyncClient;
		}
        if (wjrcbApi.isWjrcb(appId)) {
        	return wjrcbHttpAsyncClient;
		}
        if(gangZhongLvApi.isGangZhongLv(appId)){
            return ctsHttpAsyncClient;
        }
        if (cgbBankApi.isCgbBank(appId)) {
            return cgbHttpAsyncClient;
        }
		if (hsbcConfig.isHsbcApp(appId)) {
			return hsbcHttpAsyncClient;
		}
        return null;
    }

	@NotNull
	private CloseableHttpAsyncClient getCloseableHttpAsyncClient(RequestTypeEnum requestTypeEnum) {
		CloseableHttpAsyncClient httpClient = httpAsyncClient;
		if(requestTypeEnum != null && requestTypeEnum.getCode() == RequestTypeEnum.VIRTUAL.getCode()){
			httpClient = virtualHttpAsyncClient;
		}
		if(requestTypeEnum != null && requestTypeEnum.getCode() == RequestTypeEnum.ADD_CREDITS.getCode()){
			httpClient = addHttpAsyncClient;
		}
		return httpClient;
	}

	/**
	 * 判断APP能否提交任务到执行队列中
	 *
	 * @param queueKey
	 * @return
	 */
	public boolean canSubmitToAppPool(String queueKey) {
		int queueCount = 0;
		AtomicInteger running = runningStat.get(queueKey);
		if (running != null) {
			queueCount = running.intValue();
		}
		if (queueCount < MAX_APP_QUEUE) {
			return true;
		}
		return false;
	}

	/**
	 * 队列详细信息
	 *
	 * @return
	 */
	public Map<String, Object> dumpDetail() {
		Map<String, Object> map = new HashMap<>();
		for (Map.Entry<String, AtomicInteger> entry : runningStat.entrySet()) {
			if (entry.getValue().intValue() > 0) {
				map.put(entry.getKey(), entry.getValue());
			}
		}
		Map<String, Object> ret = new HashMap<>();
		ret.put("runningStat", map);
		return ret;
	}

	/**
	 * HTTP处理中的个数
	 *
	 * @return
	 */
	public int dumpSize() {
		int running = 0;
		for (AtomicInteger a : runningStat.values()) {
			running += a.get();
		}
		return running;
	}

	/**
	 * 自定义回调
	 */
	public class CallbackProcesser implements FutureCallback<HttpResponse> {

		private FutureCallback<HttpResponse> callback;
		private String queueKey;
		private HttpUriRequest request;

		public CallbackProcesser(String queueKey, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
			this.callback = callback;
			this.queueKey = queueKey;
			this.request = request;
		}

		@Override
		public void completed(HttpResponse result) {
			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.completed(result);
					return null;
				}, "AsyncHttp", "completed");
			}catch (Throwable te ){
				log.error("completed;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//
//			try {
//				callback.completed(result);
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		@Override
		public void failed(Exception ex) {

			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.failed(ex);
					return null;
				}, "AsyncHttp", "failed");
			} catch (Throwable te) {
				log.error("failed;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//			try {
//				callback.failed(ex);
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		@Override
		public void cancelled() {

			try {
				CatUtils.executeInCatTransaction(() -> {
					callback.cancelled();
					return null;
				}, "AsyncHttp", "cancelled");
			} catch (Throwable te) {
				log.error("cancelled;",te);
			}finally {
				runningStat.get(queueKey).decrementAndGet();
			}

//			try {
//				callback.cancelled();
//			} finally {
//				runningStat.get(queueKey).decrementAndGet();
//			}
		}

		public HttpUriRequest getRequest() {
			return request;
		}

		public String getQueueKey() {
			return queueKey;
		}
	}

	/**
	 * 销毁关闭
	 */
	@Override
	public void destroy() throws Exception {
		status = false;
		executorService.shutdown();
	}

	/**
	 * 初始化启动
	 */
	@Override
	public void afterPropertiesSet() throws Exception {
		executorService.execute(new Runnable() {
			@Override
			public void run() {
				while (status) {
					try{
						if (!threadPoolService.canSubmit()) {
							TimeUnit.MILLISECONDS.sleep(200);
							continue;
						}
						CallbackProcesser callbackProcesser =latterQueue.take();
						if (callbackProcesser == null) {
							continue;
						}
						final String queueKey = callbackProcesser.getQueueKey();
						final HttpUriRequest request = callbackProcesser.getRequest();
						final FutureCallback<HttpResponse> callback = callbackProcesser.callback;
						threadPoolService.submit(new Runnable() {
							@Override
							public void run() {
								execute(queueKey, request, callback, slowHttpAsyncClient);
							}
						});
					}catch (InterruptedException e) {
						log.warn("Interrupted",e);
					}catch (Exception e) {
						log.warn("",e);
					}
				}
			}
		});
	}
}
